@@ -13,11 +13,12 @@
 ##############################################################################
 """Browser widgets for items
 
-$Id: itemswidgets.py 28693 2004-12-23 21:06:23Z jim $
+$Id: itemswidgets.py 37977 2005-08-17 01:02:10Z poster $
 """
 __docformat__ = 'restructuredtext'
-
+import sets
 from xml.sax.saxutils import escape
+
 from zope.interface import implements
 from zope.i18n import translate
 from zope.schema.interfaces import ValidationError, InvalidValue
@@ -35,31 +36,35 @@ from zope.app.i18n import ZopeMessageIDFactory as _
 # For choices, we want to make the widget a view of the field and vocabulary.
 
 def ChoiceDisplayWidget(field, request):
-    return zapi.getMultiView((field, field.vocabulary), request, IDisplayWidget)
+    return zapi.getMultiAdapter((field, field.vocabulary, request),
+                                IDisplayWidget)
 
 def ChoiceInputWidget(field, request):
-    return zapi.getMultiView((field, field.vocabulary), request, IInputWidget)
+    return zapi.getMultiAdapter((field, field.vocabulary, request),
+                                IInputWidget)
 
 # for collections, we want to make the widget a view of the field and the
 # value_type.  If the value_type is None we may fall over.  We may
 # not be able to do any better than that.
 
 def CollectionDisplayWidget(field, request):
-    return zapi.getMultiView((field, field.value_type), request, IDisplayWidget)
+    return zapi.getMultiAdapter((field, field.value_type, request),
+                                IDisplayWidget)
 
 def CollectionInputWidget(field, request):
-    return zapi.getMultiView((field, field.value_type), request, IInputWidget)
+    return zapi.getMultiAdapter((field, field.value_type, request),
+                                IInputWidget)
 
 # for collections of choices, we want to make the widget a view of the field,
 # the value type, and the vocabulary.
 
 def ChoiceCollectionDisplayWidget(field, value_type, request):
-    return zapi.getMultiView(
-        (field, value_type.vocabulary), request, IDisplayWidget)
+    return zapi.getMultiAdapter((field, value_type.vocabulary, request),
+                                IDisplayWidget)
 
 def ChoiceCollectionInputWidget(field, value_type, request):
-    return zapi.getMultiView(
-        (field, value_type.vocabulary), request, IInputWidget)
+    return zapi.getMultiAdapter((field, value_type.vocabulary, request),
+                                IInputWidget)
 
 class TranslationHook(object):
     """A mixin class that provides the translation capabilities."""
@@ -95,18 +100,21 @@ class ItemsWidgetBase(TranslationHook, SimpleInputWidget):
     def __call__(self):
         """Render the widget to HTML."""
         raise NotImplementedError(
-            "__call__() must be implemented by a subclass; use _getFormValue()")
+            "__call__() must be implemented by a subclass; use _getFormValue()"
+            )
 
     def textForValue(self, term):
         """Extract a string from the `term`.
 
         The `term` must be a vocabulary tokenized term.
 
-        This can be overridden to support more complex `term` objects. The token
-        is returned here since it's the only thing known to be a string, or
-        str()able."""
+        This can be overridden to support more complex `term`
+        objects. The token is returned here since it's the only thing
+        known to be a string, or str()able.
+
+        """
         if ITitledTokenizedTerm.providedBy(term):
-            return term.title
+            return self.translate(term.title)
         return term.token
 
     def convertTokensToValues(self, tokens):
@@ -184,10 +192,19 @@ class MultiDataHelper(object):
         if not isinstance(input, list):
             input = [input]
         try:
-            return self.convertTokensToValues(input)
+            values = self.convertTokensToValues(input)
         except InvalidValue, e:
             raise ConversionError("Invalid value", e)
 
+        # All AbstractCollection fields have a `_type` attribute specifying
+        # the type of collection. Use it to generate the correct type,
+        # otherwise return a list. 
+        if hasattr(self.context, '_type'):
+            return self.context._type(values)
+        else:
+            return values
+
+
     def _getDefault(self):
         # Return the default value for this widget;
         # may be overridden by subclasses.
@@ -202,6 +219,10 @@ class MultiDataHelper(object):
 class ItemDisplayWidget(SingleDataHelper, ItemsWidgetBase):
     """Simple single-selection display that can be used in many cases."""
 
+    def __init__(self, *args, **kw):
+        ItemsWidgetBase.__init__(self, *args, **kw)
+        self.required = False
+
     _messageNoValue = message(_("item-missing-single-value-for-display"), "")
 
     def __call__(self):
@@ -217,6 +238,10 @@ class ItemDisplayWidget(SingleDataHelper, ItemsWidgetBase):
 class ItemsMultiDisplayWidget(MultiDataHelper, ItemsWidgetBase):
     """Displays a sequence of items."""
 
+    def __init__(self, *args, **kw):
+        ItemsWidgetBase.__init__(self, *args, **kw)
+        self.required = False
+
     _messageNoValue = message(
         _("vocabulary-missing-multiple-value-for-display"), "")
 
@@ -260,7 +285,6 @@ class ListDisplayWidget(ItemsMultiDisplayWidget):
     """
     tag = 'ol'
 
-
 class SetDisplayWidget(ItemsMultiDisplayWidget):
     """Display widget for unordered multi-selection fields.
 
@@ -305,8 +329,7 @@ class ItemsEditWidgetBase(SingleDataHelper, ItemsWidgetBase):
         contents.append(self._div('value', self.renderValue(value)))
         contents.append(self._emptyMarker())
 
-        return self._div(self.cssClass, "\n".join(contents),
-                         id=self.name)
+        return self._div(self.cssClass, "\n".join(contents))
 
 
     def _div(self, cssClass, contents, **kw):
@@ -377,6 +400,7 @@ class SelectWidget(ItemsEditWidgetBase):
         contents = "\n%s\n" %"\n".join(rendered_items)
         return renderElement('select',
                              name=self.name,
+                             id=self.name,
                              contents=contents,
                              size=self.size,
                              extra=self.extra)
@@ -524,6 +548,17 @@ class ItemsMultiEditWidgetBase(MultiDataHelper, ItemsEditWidgetBase):
 class MultiSelectWidget(ItemsMultiEditWidgetBase):
     """Provide a selection list for the list to be selected."""
 
+
+class MultiSelectSetWidget(MultiSelectWidget):
+    """Provide a selection list for the set to be selected."""
+
+    def _toFieldValue(self, input):
+        value = super(MultiSelectSetWidget, self)._toFieldValue(input)
+        if isinstance(value, list):
+            value = sets.Set(value)
+        return value
+
+
 class OrderedMultiSelectWidget(ItemsMultiEditWidgetBase):
     """A multi-selection widget with ordering support."""
 
@@ -531,28 +566,34 @@ class OrderedMultiSelectWidget(ItemsMultiEditWidgetBase):
 
     def choices(self):
         """Return a set of tuples (text, value) that are available."""
-        # Not all content object must necessarily support the attributes
+        # Not all content objects must necessarily support the attributes
         if hasattr(self.context.context, self.context.__name__):
-            selected_values = self.context.get(self.context.context)
+            available_values = self.context.get(self.context.context)
         else:
-            selected_values = []
+            available_values = []
         return [{'text': self.textForValue(term), 'value': term.token}
                 for term in self.vocabulary
-                if term.value not in selected_values]
-        
+                if term.value not in available_values]
+
     def selected(self):
         """Return a list of tuples (text, value) that are selected."""
-        # Not all content object must necessarily support the attributes
-        if not hasattr(self.context.context, self.context.__name__):
-            return []
+        # Get form values
+        values = self._getFormValue()
+        # Not all content objects must necessarily support the attributes
+        if hasattr(self.context.context, self.context.__name__):
+            # merge in values from content 
+            for value in self.context.get(self.context.context):
+                if value not in values:
+                    values.append(value)
+
         terms = [self.vocabulary.getTerm(value)
-                 for value in self.context.get(self.context.context)]
+                 for value in values]
         return [{'text': self.textForValue(term), 'value': term.token}
                 for term in terms]
-    
+
     def __call__(self):
         return self.template()
-    
+
 
 class MultiCheckBoxWidget(ItemsMultiEditWidgetBase):
     """Provide a list of checkboxes that provide the choice for the list."""
@@ -588,4 +629,3 @@ class MultiCheckBoxWidget(ItemsMultiEditWidgetBase):
                              value=value,
                              checked="checked")
         return self._joinButtonToMessageTemplate %(elem, text)
-
