@@ -13,7 +13,7 @@
 ##############################################################################
 """Pagelet tests
 
-$Id: test_pagelet.py 28396 2004-11-08 13:57:54Z rogerineichen $
+$Id: test_pagelet.py 29143 2005-02-14 22:43:16Z srichter $
 """
 __docformat__ = 'restructuredtext'
 
@@ -26,8 +26,7 @@ from zope.security.checker import defineChecker
 from zope.testing.doctestunit import DocTestSuite
 from zope.testing.doctestunit import DocFileSuite
 
-from zope.app.tests import placelesssetup, ztapi
-from zope.app.tests import setup
+from zope.app.testing import placelesssetup, ztapi, setup
 
 
 
@@ -42,4 +41,4 @@ def test_suite():
         ))
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='test_suite')
\ No newline at end of file
+    unittest.main(defaultTest='test_suite')
