@@ -13,7 +13,7 @@
 ##############################################################################
 """Zope Publication Tests
 
-$Id: test_zopepublication.py 28663 2004-12-21 03:34:46Z srichter $
+$Id: test_zopepublication.py 30927 2005-06-25 16:58:29Z philikon $
 """
 import unittest
 import sys
@@ -22,12 +22,11 @@ from cStringIO import StringIO
 from persistent import Persistent
 from ZODB.DB import DB
 from ZODB.DemoStorage import DemoStorage
+import transaction
 
 from zope.interface.verify import verifyClass
 from zope.interface import implements, classImplements, implementedBy
 from zope.i18n.interfaces import IUserPreferredCharsets
-from zope.component import getGlobalServices
-from zope.component.interfaces import IServiceService
 from zope.component.exceptions import ComponentLookupError
 from zope.publisher.base import TestPublication, TestRequest
 from zope.publisher.http import IHTTPRequest, HTTPCharsets
@@ -35,14 +34,13 @@ from zope.publisher.interfaces import IRequest, IPublishTraverse
 from zope.publisher.browser import BrowserResponse
 from zope.security import simplepolicies
 from zope.security.management import setSecurityPolicy, queryInteraction
+from zope.security.management import endInteraction
 
 from zope.app import zapi
-from zope.app.tests.placelesssetup import PlacelessSetup
-from zope.app.tests import setup
-from zope.app.tests import ztapi
+from zope.app.testing.placelesssetup import PlacelessSetup
+from zope.app.testing import setup, ztapi
 
 from zope.app.error.interfaces import IErrorReportingUtility
-from zope.app.servicenames import Authentication
 from zope.app.location.interfaces import ILocation
 from zope.app.traversing.interfaces import IPhysicallyLocatable
 from zope.app.security.principalregistry import principalRegistry
@@ -50,6 +48,7 @@ from zope.app.security.interfaces import IUnauthenticatedPrincipal, IPrincipal
 from zope.app.publication.zopepublication import ZopePublication
 from zope.app.folder import Folder, rootFolder
 from zope.app.location import Location
+from zope.app.security.interfaces import IAuthenticationUtility
 
 class Principal(object):
     implements(IPrincipal)
@@ -92,26 +91,6 @@ class ErrorReportingUtility(object):
     def raising(self, info, request=None):
         self.exceptions.append([info, request])
 
-
-class UtilityService(object):
-
-    utility = None
-
-    def getUtility(self, interface, name=''):
-        return self.utility
-
-
-class ServiceManager(object):
-    implements(IServiceService) # a dirty lie
-
-    def __init__(self, utils):
-        self.utils = utils
-
-    def getService(self, name):
-        if name == 'Utilities':
-            return self.utils
-        raise ComponentLookupError(name)
-
 class LocatableObject(Location):
 
     def foo(self):
@@ -142,7 +121,7 @@ class BasePublicationTests(PlacelessSetup, unittest.TestCase):
             from zope.app.folder import rootFolder
             app = rootFolder()
             root[ZopePublication.root_name] = app
-            get_transaction().commit()
+            transaction.commit()
 
         connection.close()
         self.app = app
@@ -163,7 +142,6 @@ class BasePublicationTests(PlacelessSetup, unittest.TestCase):
         self.publication = ZopePublication(self.db)
 
     def tearDown(self):
-        setSecurityPolicy(self.policy) # XXX still needed?
         super(BasePublicationTests, self).tearDown()
 
     def testInterfacesVerify(self):
@@ -235,7 +213,7 @@ class ZopePublicationErrorHandling(BasePublicationTests):
             '  Error while reporting an error to the Error Reporting utility')
 
         # Here we got a single log record, because we havdn't
-        # installed an error reporting service.  That's OK.
+        # installed an error reporting utility.  That's OK.
 
         handler.uninstall()
         self.out.seek(0)
@@ -277,7 +255,7 @@ class ZopePublicationErrorHandling(BasePublicationTests):
 
         # Now, since the view was a system error view, we should have
         # a log entry for the E2 error (as well as the missing
-        # error reporting service).
+        # error reporting utility).
         self.assertEqual(
             str(handler),
             'SiteError ERROR\n'
@@ -366,7 +344,7 @@ class ZopePublicationErrorHandling(BasePublicationTests):
         self.assertEqual(self.request.response._cookies, {})
 
     def testAbortOrCommitTransaction(self):
-        txn = get_transaction()
+        txn = transaction.get()
         try:
             raise Exception
         except:
@@ -374,14 +352,12 @@ class ZopePublicationErrorHandling(BasePublicationTests):
         self.publication.handleException(
             self.object, self.request, sys.exc_info(), retry_allowed=False)
         # assert that we get a new transaction
-        self.assert_(txn is not get_transaction())    
+        self.assert_(txn is not transaction.get())    
 
     def testAbortTransactionWithErrorReportingUtility(self):
-        # provide our fake error logging service
-        sm = getGlobalServices()
-        utils = sm.getService('Utilities')
-        utils.provideUtility(IErrorReportingUtility,
-                             ErrorReportingUtility())
+        # provide our fake error reporting utility
+        sm = zapi.getGlobalSiteManager()
+        sm.provideUtility(IErrorReportingUtility, ErrorReportingUtility())
 
         class FooError(Exception):
             pass
@@ -398,8 +374,8 @@ class ZopePublicationErrorHandling(BasePublicationTests):
         new_txn_info = self.db.undoInfo()[0]
         self.assertEqual(last_txn_info, new_txn_info)
 
-        # instead, we expect a message in our logging service
-        error_log = utils.getUtility(IErrorReportingUtility)
+        # instead, we expect a message in our logging utility
+        error_log = zapi.getUtility(IErrorReportingUtility)
         self.assertEqual(len(error_log.exceptions), 1)
         error_info, request = error_log.exceptions[0]
         self.assertEqual(error_info[0], FooError)
@@ -410,32 +386,34 @@ class ZopePublicationErrorHandling(BasePublicationTests):
 class ZopePublicationTests(BasePublicationTests):
 
     def testPlacefulAuth(self):
+        setup.setUpTraversal()
+        setup.setUpSiteManagerLookup()
         principalRegistry.defineDefaultPrincipal('anonymous', '')
 
         root = self.db.open().root()
         app = root[ZopePublication.root_name]
-        app['f1'] = Folder()
+        app['f1'] = rootFolder()
         f1 = app['f1']
         f1['f2'] = Folder()
-        utilservice1 = UtilityService()
-        utilservice1.utility = AuthUtility1()
-        f1.setSiteManager(ServiceManager(utilservice1))
+        sm1 = setup.createSiteManager(f1)
+        setup.addUtility(sm1, '', IAuthenticationUtility, AuthUtility1())
+        
         f2 = f1['f2']
-        utilservice2 = UtilityService()
-        utilservice2.utility = AuthUtility2()
-        f2.setSiteManager(ServiceManager(utilservice2))
-        get_transaction().commit()
-
+        sm2 = setup.createSiteManager(f2)
+        setup.addUtility(sm2, '', IAuthenticationUtility, AuthUtility2())
+        transaction.commit()
+        
+        
         from zope.app.container.interfaces import ISimpleReadContainer
         from zope.app.container.traversal import ContainerTraverser
-
+        
         ztapi.provideView(ISimpleReadContainer, IRequest, IPublishTraverse,
                           '', ContainerTraverser)
-
+        
         from zope.app.folder.interfaces import IFolder
         from zope.security.checker import defineChecker, InterfaceChecker
         defineChecker(Folder, InterfaceChecker(IFolder))
-
+        
         self.publication.beforeTraversal(self.request)
         self.assertEqual(list(queryInteraction().participations),
                          [self.request])
@@ -456,13 +434,13 @@ class ZopePublicationTests(BasePublicationTests):
 
     def testTransactionCommitAfterCall(self):
         root = self.db.open().root()
-        txn = get_transaction()
+        txn = transaction.get()
         # we just need a change in the database to make the
         # transaction notable in the undo log
         root['foo'] = object()
         last_txn_info = self.db.undoInfo()[0]
         self.publication.afterCall(self.request, self.object)
-        self.assert_(txn is not get_transaction())
+        self.assert_(txn is not transaction.get())
         new_txn_info = self.db.undoInfo()[0]
         self.failIfEqual(last_txn_info, new_txn_info)
 
@@ -506,8 +484,8 @@ class ZopePublicationTests(BasePublicationTests):
 
         set = []
         clear = []
-        ztapi.handle([IBeforeTraverseEvent], set.append)
-        ztapi.handle([IEndRequestEvent], clear.append)
+        ztapi.subscribe([IBeforeTraverseEvent], None, set.append)
+        ztapi.subscribe([IEndRequestEvent], None, clear.append)
 
         ob = object()
 
