@@ -13,11 +13,10 @@
 ##############################################################################
 """ProcessDefinition registration adding view
 
-$Id: definition.py 27548 2004-09-16 14:09:29Z eckart $
+$Id: definition.py 29143 2005-02-14 22:43:16Z srichter $
 """
-
 from zope.app.traversing.api import traverse
-from zope.app.registration.interfaces import IRegistered
+from zope.app.component.interfaces.registration import IRegistered
 from zope.app.workflow.interfaces import IProcessDefinitionImportHandler
 from zope.app.workflow.interfaces import IProcessDefinitionExportHandler
 
