@@ -13,7 +13,7 @@
 ##############################################################################
 """Interface Package Interfaces
 
-$Id: interfaces.py 28959 2005-01-26 16:49:06Z frerich $
+$Id: interfaces.py 29118 2005-02-11 17:44:16Z jim $
 """
 __docformat__ = 'restructuredtext'
 
@@ -418,6 +418,14 @@ class IInterfaceDeclaration(Interface):
         instances of ``A`` and ``B`` provide.
         """
 
+    def implementer(*interfaces):
+        """Create a decorator for declaring interfaces implemented by a facory
+
+        A callable is returned that makes an implements declaration on
+        objects passed to it.
+        
+        """
+
     def classImplementsOnly(class_, *interfaces):
         """Declare the only interfaces implemented by instances of a class
 
