@@ -1,6 +1,5 @@
 # -*- coding: ISO-8859-1 -*-
 ##############################################################################
-#
 # Copyright (c) 2002 Zope Corporation and Contributors.
 # All Rights Reserved.
 #
@@ -14,12 +13,12 @@
 ##############################################################################
 """Schema Fields
 
-$Id: _field.py 28984 2005-01-30 19:13:23Z gintautasm $
+$Id: _field.py 37565 2005-07-29 20:46:23Z srichter $
 """
 __docformat__ = 'restructuredtext'
 import warnings
 import re
-from datetime import datetime, date
+from datetime import datetime, date, timedelta
 from sets import Set as SetType
 
 from zope.interface import classImplements, implements, directlyProvides
@@ -29,12 +28,12 @@ from zope.schema.interfaces import IField
 from zope.schema.interfaces import IMinMaxLen, IText, ITextLine
 from zope.schema.interfaces import ISourceText
 from zope.schema.interfaces import IInterfaceField
-from zope.schema.interfaces import IBytes, IASCII, IBytesLine
+from zope.schema.interfaces import IBytes, IASCII, IBytesLine, IASCIILine
 from zope.schema.interfaces import IBool, IInt, IFloat, IDatetime
 from zope.schema.interfaces import IChoice, ITuple, IList, ISet, IDict
-from zope.schema.interfaces import IPassword, IObject, IDate
+from zope.schema.interfaces import IPassword, IObject, IDate, ITimedelta
 from zope.schema.interfaces import IURI, IId, IFromUnicode
-from zope.schema.interfaces import ISource, IVocabulary
+from zope.schema.interfaces import ISource, IBaseVocabulary
 
 from zope.schema.interfaces import ValidationError, InvalidValue
 from zope.schema.interfaces import WrongType, WrongContainedType, NotUnique
@@ -134,6 +133,14 @@ class BytesLine(Bytes):
         # TODO: we should probably use a more general definition of newlines
         return '\n' not in value
 
+class ASCIILine(ASCII):
+    __doc__ = IASCIILine.__doc__
+
+    implements(IASCIILine)
+
+    def constraint(self, value):
+        # TODO: we should probably use a more general definition of newlines
+        return '\n' not in value
 
 class Float(Orderable, Field):
     __doc__ = IFloat.__doc__
@@ -170,6 +177,11 @@ class Date(Orderable, Field):
     implements(IDate)
     _type = date
 
+class Timedelta(Orderable, Field):
+    __doc__ = ITimedelta.__doc__
+    implements(ITimedelta)
+    _type = timedelta
+
 class Choice(Field):
     """Choice fields can have a value found in a constant or dynamic set of
     values given by the field definition.
@@ -180,7 +192,7 @@ class Choice(Field):
         """Initialize object."""
         if vocabulary is not None:
             assert (isinstance(vocabulary, basestring)
-                    or IVocabulary.providedBy(vocabulary))
+                    or IBaseVocabulary.providedBy(vocabulary))
             assert source is None, (
                 "You cannot specify both source and vocabulary.")
         elif source is not None:
@@ -271,19 +283,19 @@ def _validate_sequence(value_type, value, errors=None):
 
     To illustrate, we'll use a text value type. All values must be unicode.
 
-	    >>> field = TextLine(required=True)
+            >>> field = TextLine(required=True)
 
-	To validate a sequence of various values:
+        To validate a sequence of various values:
 
-	    >>> errors = _validate_sequence(field, ('foo', u'bar', 1))
-	    >>> errors
-	    [foo <type 'unicode'>, 1 <type 'unicode'>]
+            >>> errors = _validate_sequence(field, ('foo', u'bar', 1))
+            >>> errors
+            [foo <type 'unicode'>, 1 <type 'unicode'>]
 
-	The only valid value in the sequence is the second item. The others
-	generated errors.
+        The only valid value in the sequence is the second item. The others
+        generated errors.
 
-	We can use the optional errors argument to collect additional errors
-	for a new sequence:
+        We can use the optional errors argument to collect additional errors
+        for a new sequence:
 
         >>> errors = _validate_sequence(field, (2, u'baz'), errors)
         >>> errors
