@@ -13,11 +13,11 @@
 ******************************************************************************
 Security Proxy Implementation
 
-$Id: _proxy.c 26665 2004-07-21 22:49:38Z jim $
+$Id: _proxy.c 37882 2005-08-11 21:53:00Z fdrake $
 */
 
 #include <Python.h>
-#include "proxy.h"
+#include "zope.proxy/proxy.h"
 
 static PyObject *__class__str = 0, *__name__str = 0, *__module__str = 0;
 
