@@ -1,4 +1,4 @@
-#!/usr/bin/env python2.3
+#!/usr/bin/env python
 #
 # SchoolTool - common information systems platform for school administration
 # Copyright (c) 2003 Shuttleworth Foundation
@@ -47,8 +47,11 @@ Options:
   -v                    verbose (print dots for each test run)
   -vv                   very verbose (print test names)
   -q                    quiet (do not print anything on success)
+  -c                    colorize output (assumes dark background)
+  -C                    colorize output (assumes bright background)
   -w                    enable warnings about omitted test cases
   -d                    invoke pdb when an exception occurs
+  -1                    report only the first failure in doctests
   -p                    show progress bar (can be combined with -v or -vv)
   -u                    select unit tests (default)
   -f                    select functional tests
@@ -115,9 +118,11 @@ class Options:
     quiet = 0                   # do not print anything on success (-q)
     warn_omitted = False        # produce warnings when a test case is
                                 # not included in a test suite (-w)
+    first_doctest_failure = False # report first doctest failure (-1)
     print_import_time = True    # print time taken to import test modules
                                 # (currently hardcoded)
     progress = False            # show running progress (-p)
+    colorizer = None            # colorize output (-c)
     coverage = False            # produce coverage reports (--coverage)
     coverdir = 'coverage'       # where to put them (currently hardcoded)
     immediate_errors = True     # show tracebacks twice (--immediate-errors,
@@ -126,7 +131,7 @@ class Options:
 
 
 def compile_matcher(regex):
-    """Returns a function that takes one argument and returns True or False.
+    """Return a function that takes one argument and returns True or False.
 
     Regex is a regular expression.  Empty regex matches everything.  There
     is one expression: if the regex starts with "!", the meaning of it is
@@ -163,18 +168,18 @@ def walk_with_symlinks(top, func, arg):
 
 
 def get_test_files(cfg):
-    """Returns a list of test module filenames."""
+    """Return a list of test module filenames."""
     matcher = compile_matcher(cfg.pathname_regex)
-    results = []
+    allresults = []
     test_names = []
-    if cfg.unit_tests:
-        test_names.append('tests')
     if cfg.functional_tests:
         test_names.append('ftests')
+    if cfg.unit_tests:
+        test_names.append('tests')
     baselen = len(cfg.basedir) + 1
     def visit(ignored, dir, files):
         # Ignore files starting with a dot.
-        # Do not not descend into subdirs containing with a dot.
+        # Do not not descend into subdirs containing a dot.
         remove = []
         for idx, file in enumerate(files):
             if file.startswith('.'):
@@ -184,13 +189,12 @@ def get_test_files(cfg):
         remove.reverse()
         for idx in remove:
             del files[idx]
-        # Look for tests.py and/or ftests.py
-        if os.path.basename(dir) not in test_names:
-            for name in test_names:
-                if name + '.py' in files:
-                    path = os.path.join(dir, name + '.py')
-                    if matcher(path[baselen:]):
-                        results.append(path)
+        # Skip non-test directories, but look for tests.py and/or ftests.py
+        if os.path.basename(dir) != test_name:
+            if test_name + '.py' in files:
+                path = os.path.join(dir, test_name + '.py')
+                if matcher(path[baselen:]):
+                    results.append(path)
             return
         if '__init__.py' not in files:
             print >> sys.stderr, "%s is not a package" % dir
@@ -204,14 +208,19 @@ def get_test_files(cfg):
         walker = walk_with_symlinks
     else:
         walker = os.path.walk
-    for dir in cfg.search_in:
-        walker(dir, visit, None)
-    results.sort()
-    return results
+
+    for test_name in test_names:
+        results = []
+        for dir in cfg.search_in:
+            walker(dir, visit, None)
+        results.sort()
+        allresults += results
+
+    return allresults
 
 
 def import_module(filename, cfg, tracer=None):
-    """Imports and returns a module."""
+    """Import and return a module."""
     filename = os.path.splitext(filename)[0]
     modname = filename[len(cfg.basedir):].replace(os.path.sep, '.')
     if modname.startswith('.'):
@@ -227,7 +236,7 @@ def import_module(filename, cfg, tracer=None):
 
 
 def filter_testsuite(suite, matcher, level=None):
-    """Returns a flattened list of test cases that match the given matcher."""
+    """Return a flattened list of test cases that match the given matcher."""
     if not isinstance(suite, unittest.TestSuite):
         raise TypeError('not a TestSuite', suite)
     results = []
@@ -245,7 +254,7 @@ def filter_testsuite(suite, matcher, level=None):
 
 
 def get_all_test_cases(module):
-    """Returns a list of all test case classes defined in a given module."""
+    """Return a list of all test case classes defined in a given module."""
     results = []
     for name in dir(module):
         if not name.startswith('Test'):
@@ -258,7 +267,7 @@ def get_all_test_cases(module):
 
 
 def get_test_classes_from_testsuite(suite):
-    """Returns a set of test case classes used in a test suite."""
+    """Return a set of test case classes used in a test suite."""
     if not isinstance(suite, unittest.TestSuite):
         raise TypeError('not a TestSuite', suite)
     results = Set()
@@ -272,7 +281,7 @@ def get_test_classes_from_testsuite(suite):
 
 
 def get_test_cases(test_files, cfg, tracer=None):
-    """Returns a list of test cases from a given list of test modules."""
+    """Return a list of test cases from a given list of test modules."""
     matcher = compile_matcher(cfg.test_regex)
     results = []
     startTime = time.time()
@@ -312,13 +321,14 @@ def get_test_cases(test_files, cfg, tracer=None):
     timeTaken = float(stopTime - startTime)
     if cfg.print_import_time:
         nmodules = len(test_files)
-        print "Imported %d modules in %.3fs" % (nmodules, timeTaken)
+        plural = (nmodules != 1) and 's' or ''
+        print "Imported %d module%s in %.3fs" % (nmodules, plural, timeTaken)
         print
     return results
 
 
 def get_test_hooks(test_files, cfg, tracer=None):
-    """Returns a list of test hooks from a given list of test modules."""
+    """Return a list of test hooks from a given list of test modules."""
     results = []
     dirs = Set(map(os.path.dirname, test_files))
     for dir in list(dirs):
@@ -355,28 +365,283 @@ def extract_tb(tb, limit=None):
         tb = tb.tb_next
     return list
 
-def format_exception(etype, value, tb, limit=None, basedir=None):
+
+
+colorcodes = {'gray': 0, 'red': 1, 'green': 2, 'yellow': 3,
+              'blue': 4, 'magenta': 5, 'cyan': 6, 'white': 7}
+
+dark_colormap = { # Color scheme for dark backgrounds
+            'fail': 'red',
+            'pass': 'green',
+            'count': 'white',
+            'title': 'white',
+            'separator': 'dark white',
+            'longtestname': 'yellow',
+            'filename': 'dark green',
+            'lineno': 'green',
+            'testname': 'dark yellow',
+            'excname': 'red',
+            'excstring': 'yellow',
+            'tbheader': 'dark white',
+            'doctest_ignored': 'gray',
+            'doctest_title': 'dark white',
+            'doctest_code': 'yellow',
+            'doctest_expected': 'green',
+            'doctest_got': 'red',
+            'diff_expected': 'red',
+            'diff_actual': 'green',
+            'diff_context': 'dark white',
+            'diff_inline': 'gray'}
+
+light_colormap = { # Color scheme for light backgrounds
+            'fail': 'red',
+            'pass': 'dark green',
+            'count': 'dark green',
+            'title': 'red',
+            'separator': 'dark white',
+            'longtestname': 'dark red',
+            'filename': 'dark green',
+            'lineno': 'dark magenta',
+            'testname': 'dark yellow',
+            'excname': 'red',
+            'excstring': 'dark yellow',
+            'tbheader': 'gray',
+            'doctest_ignored': 'dark white',
+            'doctest_title': 'gray',
+            'doctest_code': 'dark blue',
+            'doctest_expected': 'dark green',
+            'doctest_got': 'dark red',
+            'diff_expected': 'dark red',
+            'diff_actual': 'dark green',
+            'diff_context': 'dark gray',
+            'diff_inline': 'dark magenta'}
+
+
+class Colorizer(object):
+
+    def __init__(self, colormap):
+        self.colormap = colormap
+
+    def colorize(self, texttype, text):
+        """Colorize text by ANSI escape codes in a color provided in colormap."""
+        color = self.colormap[texttype]
+        if color.startswith('dark '):
+            light = 0
+            color = color[len('dark '):] # strip the 'dark' prefix
+        else:
+            light = 1
+        code = 30 + colorcodes[color]
+        return '\033[%d;%dm' % (light, code) + text + '\033[0m'
+
+    def colorize_ndiff(self, lines):
+        """Colorize ndiff output.
+
+        Returns a new sequence of colored strings.
+
+        `lines` is a sequence of strings.
+
+        Typical input:
+
+              Some context lines
+            - This line was removed
+              Some context
+            + This line was added
+              Some context
+            - This line esd chnged
+            ?           ^^^     -
+            + This line was changd
+            ?           ^^^   + 
+              Some context
+
+        """
+        result = []
+        for line in lines:
+            if line.startswith('    -'):
+                result.append(self.colorize('diff_expected', line))
+            elif line.startswith('    +'):
+                result.append(self.colorize('diff_actual', line))
+            elif line.startswith('    ?'):
+                result.append(self.colorize('diff_inline', line))
+            else:
+                result.append(self.colorize('diff_context', line))
+        return result
+
+    def colorize_zope_doctest_output(self, lines):
+        """Colorize output formatted by the doctest engine included with Zope 3.
+
+        Returns a new sequence of colored strings.
+
+        `lines` is a sequence of strings.
+
+        The typical structure of the doctest output looks either like this:
+
+            File "...", line 123, in foo.bar.baz.doctest_quux
+            Failed example:
+                f(2, 3)
+            Expected:
+                6
+            Got:
+                5
+
+        Or, if an exception has occured, like this:
+
+            File "...", line 123, in foo.bar.baz.doctest_quux
+            Failed example:
+                f(2, 3)
+            Exception raised:
+                Traceback (most recent call last):
+                  File "...", line 123, in __init__
+                    self.do_something(a, b, c)
+                  File "...", line ...
+                    ...
+                FooError: something bad happened
+
+        If some assumption made by this function is not met, the original sequence
+        is returned without any modifications.
+        """
+        # XXX bug: doctest may report several failures in one test, they are
+        #          separated by a horizontal dash line.  Only the first one of
+        #          them is now colorized properly.
+        header = lines[0]
+        if not header.startswith('File "'):
+            return lines # not a doctest failure report?
+
+        # Dissect the header in a rather nasty way.
+        header = header[len('File "'):]
+        fn_end = header.find('"')
+        if fn_end == -1:
+            return lines
+        filename = header[:fn_end]
+        header = header[fn_end+len('", line '):]
+        parts = header.split(', in ')
+        if len(parts) != 2:
+            lineno = header
+            filename = self.colorize('filename', filename)
+            lineno = self.colorize('lineno', lineno)
+            result = ['File "%s", line %s' % (filename, lineno)]
+        else:
+            lineno, testname = parts
+            filename = self.colorize('filename', filename)
+            lineno = self.colorize('lineno', lineno)
+            testname = self.colorize('testname', testname)
+            result = ['File "%s", line %s, in %s' % (filename, lineno, testname)]
+
+        # Colorize the 'Failed example:' section.
+        if lines[1] != 'Failed example:':
+            return lines
+        result.append(self.colorize('doctest_title', lines[1]))
+        remaining = lines[2:]
+        terminators = ['Expected:', 'Expected nothing', 'Exception raised:',
+                       'Differences (ndiff with -expected +actual):']
+        while remaining and remaining[0] not in terminators:
+            line = remaining.pop(0)
+            result.append(self.colorize('doctest_code', line))
+        if not remaining:
+            return lines
+
+        if remaining[0] in ('Expected:', 'Expected nothing'):
+            result.append(self.colorize('doctest_title', remaining.pop(0))) # Expected:
+            while remaining and remaining[0] not in ('Got:', 'Got nothing'):
+                line = remaining.pop(0)
+                result.append(self.colorize('doctest_expected', line))
+            if not remaining or remaining[0] not in ('Got:', 'Got nothing'):
+                return lines
+            result.append(self.colorize('doctest_title', remaining.pop(0))) # Got:
+            while remaining:
+                line = remaining.pop(0)
+                result.append(self.colorize('doctest_got', line))
+        elif remaining[0] == 'Exception raised:':
+            result.append(self.colorize('doctest_title', remaining.pop(0))) # E. raised:
+            while remaining:
+                line = remaining.pop(0)
+                # TODO: Scrape and colorize the traceback.
+                result.append(self.colorize('doctest_got', line))
+        elif remaining[0] == 'Differences (ndiff with -expected +actual):':
+            result.append(self.colorize('doctest_title', remaining.pop(0))) # E. raised:
+            result.extend(self.colorize_ndiff(remaining))
+        else:
+            return lines
+
+        return result
+
+    def colorize_exception_only(self, lines):
+        """Colorize result of traceback.format_exception_only."""
+        if len(lines) > 1:
+            return lines # SyntaxError?  We won't deal with that for now.
+        lines = lines[0].splitlines()
+
+        # First, colorize the first line, which usually contains the name
+        # and the string of the exception.
+        result = []
+        doctest = 'Failed doctest test for' in lines[0]
+        # TODO: We only deal with the output from Zope 3's doctest module.
+        #       A colorizer for the Python's doctest module would be nice too.
+        if doctest:
+            # If we have a doctest, we do not care about this header.  All the
+            # interesting things are below, formatted by the doctest runner.
+            for lineno in range(4):
+                result.append(self.colorize('doctest_ignored', lines[lineno]))
+            beef = self.colorize_zope_doctest_output(lines[4:])
+            result.extend(beef)
+            return '\n'.join(result)
+        else:
+            # A simple exception.  Try to colorize the first row, leave others be.
+            excline = lines[0].split(': ', 1)
+            if len(excline) == 2:
+                excname = self.colorize('excname', excline[0])
+                excstring = self.colorize('excstring', excline[1])
+                result.append('%s: %s' % (excname, excstring))
+            else:
+                result.append(self.colorize('excstring', lines[0]))
+            result.extend(lines[1:])
+            return '\n'.join(result)
+
+
+def format_exception(etype, value, tb, limit=None, basedir=None,
+                     colorizer=None):
     """Improved version of traceback.format_exception.
 
     Includes Zope-specific extra information in tracebacks.
+
+    If colorizer is not None, it is used to colorize the output.
     """
+    color = (colorizer is not None)
+    if color:
+        colorize = colorizer.colorize
+    # Show stack trace.
     list = []
     if tb:
         list = ['Traceback (most recent call last):\n']
+        if color:
+            list[0] = colorize('tbheader', list[0])
         w = list.append
 
         for filename, lineno, name, locals in extract_tb(tb, limit):
-            w('  File "%s", line %s, in %s\n' % (filename, lineno, name))
             line = linecache.getline(filename, lineno)
-            if line:
-                w('    %s\n' % line.strip())
+            if color and 'zope/testing/doctest.py' not in filename:
+                filename = colorize('filename', filename)
+                lineno = colorize('lineno', str(lineno))
+                name = colorize('testname', name)
+                w('  File "%s", line %s, in %s\n' % (filename, lineno, name))
+                if line:
+                    w('    %s\n' % line.strip())
+            elif color:
+                s = '  File "%s", line %s, in %s\n' % (filename, lineno, name)
+                w(colorize('doctest_ignored', s))
+                if line:
+                    w('    %s\n' % colorize('doctest_ignored', line.strip()))
+            else:
+                w('  File "%s", line %s, in %s\n' % (filename, lineno, name))
+                if line:
+                    w('    %s\n' % line.strip())
+
             tb_info = locals.get('__traceback_info__')
             if tb_info is not None:
                 w('  Extra information: %s\n' % repr(tb_info))
             tb_supplement = locals.get('__traceback_supplement__')
             if tb_supplement is not None:
                 tb_supplement = tb_supplement[0](*tb_supplement[1:])
-                # XXX these should be hookable
+                # TODO these should be hookable
                 from zope.tales.tales import TALESTracebackSupplement
                 from zope.pagetemplate.pagetemplate \
                         import PageTemplateTracebackSupplement
@@ -395,7 +660,13 @@ def format_exception(etype, value, tb, limit=None, basedir=None):
                     w('  Expression: %s\n' % tb_supplement.expression)
                 else:
                     w('  __traceback_supplement__ = %r\n' % (tb_supplement, ))
-    list += traceback.format_exception_only(etype, value)
+
+    # Add the representation of the exception itself.
+    lines = traceback.format_exception_only(etype, value)
+    if color:
+        lines = colorizer.colorize_exception_only(lines)
+    list.extend(lines)
+
     return list
 
 
@@ -466,24 +737,38 @@ class CustomTestResult(unittest._TextTestResult):
         return s
 
     def printErrors(self):
+        w = self.stream.writeln
         if self.cfg.progress and not (self.dots or self.showAll):
-            self.stream.writeln()
+            w()
         if self.cfg.immediate_errors and (self.errors or self.failures):
-            self.stream.writeln(self.separator1)
-            self.stream.writeln("Tests that failed")
-            self.stream.writeln(self.separator2)
+            if self.cfg.colorizer is not None:
+                w(self.cfg.colorizer.colorize('separator', self.separator1))
+                w(self.cfg.colorizer.colorize('title', "Tests that failed"))
+                w(self.cfg.colorizer.colorize('separator', self.separator2))
+            else:
+                w(self.separator1)
+                w("Tests that failed")
+                w(self.separator2)
         self.__super_printErrors()
 
     def formatError(self, err):
-        return "".join(format_exception(basedir=self.cfg.basedir, *err))
+        return "".join(format_exception(basedir=self.cfg.basedir,
+                                        colorizer=self.cfg.colorizer, *err))
 
     def printTraceback(self, kind, test, err):
-        self.stream.writeln()
-        self.stream.writeln(self.separator1)
-        self.stream.writeln("%s: %s" % (kind, self.getDescription(test)))
-        self.stream.writeln(self.separator2)
-        self.stream.writeln(self.formatError(err))
-        self.stream.writeln()
+        w = self.stream.writeln
+        if self.cfg.colorizer is not None:
+            c = self.cfg.colorizer.colorize
+        else:
+            c = lambda texttype, text: text
+        w()
+        w(c('separator', self.separator1))
+        kind = c('fail', kind)
+        description = c('longtestname', self.getDescription(test))
+        w("%s: %s" % (kind, description))
+        w(c('separator', self.separator2))
+        w(self.formatError(err))
+        w()
 
     def addFailure(self, test, err):
         if self.cfg.immediate_errors:
@@ -531,6 +816,10 @@ class CustomTestRunner(unittest.TextTestRunner):
         """Run the given test case or test suite."""
         if self.count is None:
             self.count = test.countTestCases()
+        if self.cfg.colorizer is not None:
+            c = self.cfg.colorizer.colorize
+        else:
+            c = lambda texttype, text: text
         result = self._makeResult()
         startTime = time.time()
         test(result)
@@ -539,21 +828,25 @@ class CustomTestRunner(unittest.TextTestRunner):
         result.printErrors()
         run = result.testsRun
         if not self.cfg.quiet:
-            self.stream.writeln(result.separator2)
-            self.stream.writeln("Ran %d test%s in %.3fs" %
-                                (run, run != 1 and "s" or "", timeTaken))
+            self.stream.writeln(c('separator', result.separator2))
+            run_str = c('count', str(run))
+            time_str = c('count', '%.3f' % timeTaken)
+            self.stream.writeln("Ran %s test%s in %ss" %
+                                (run_str, run != 1 and "s" or "", time_str))
             self.stream.writeln()
         if not result.wasSuccessful():
-            self.stream.write("FAILED (")
+            self.stream.write(c('fail', "FAILED"))
+
             failed, errored = map(len, (result.failures, result.errors))
             if failed:
-                self.stream.write("failures=%d" % failed)
+                self.stream.write(" (failures=%s" % c('count', str(failed)))
             if errored:
                 if failed: self.stream.write(", ")
-                self.stream.write("errors=%d" % errored)
+                else: self.stream.write("(")
+                self.stream.write("errors=%s" % c('count', str(errored)))
             self.stream.writeln(")")
         elif not self.cfg.quiet:
-            self.stream.writeln("OK")
+            self.stream.writeln(c('pass', "OK"))
         return result
 
     def _makeResult(self):
@@ -592,7 +885,7 @@ def main(argv):
 
     # Option processing
     try:
-        opts, args = getopt.gnu_getopt(argv[1:], 'hvpqufwd',
+        opts, args = getopt.gnu_getopt(argv[1:], 'hvpcCqufwd1s:',
                                ['list-files', 'list-tests', 'list-hooks',
                                 'level=', 'all-levels', 'coverage',
                                 'search-in=', 'immediate-errors',
@@ -611,6 +904,10 @@ def main(argv):
         elif k == '-p':
             cfg.progress = True
             cfg.quiet = False
+        elif k == '-c':
+            cfg.colorizer = Colorizer(dark_colormap)
+        elif k == '-C':
+            cfg.colorizer = Colorizer(light_colormap)
         elif k == '-q':
             cfg.verbosity = 0
             cfg.progress = False
@@ -623,6 +920,8 @@ def main(argv):
             cfg.postmortem = True
         elif k == '-w':
             cfg.warn_omitted = True
+        elif k == '-1':
+            cfg.first_doctest_failure = True
         elif k == '--list-files':
             cfg.list_files = True
             cfg.run_tests = False
@@ -643,7 +942,7 @@ def main(argv):
                 return 1
         elif k == '--all-levels':
             cfg.level = None
-        elif k == '--search-in':
+        elif k in ('-s', '--search-in'):
             dir = os.path.abspath(v)
             if not dir.startswith(cfg.basedir):
                 print >> sys.stderr, ('%s: argument to --search-in (%s) must'
@@ -673,8 +972,9 @@ def main(argv):
     if not cfg.search_in:
         cfg.search_in = (cfg.basedir, )
 
-    # Do not print "Imported %d modules in %.3fs" if --list-XXX was specified
-    if cfg.list_tests or cfg.list_hooks or cfg.list_files:
+    # Do not print "Imported %d modules in %.3fs" if --list-* was specified
+    # or if quiet mode is enabled.
+    if cfg.quiet or cfg.list_tests or cfg.list_hooks or cfg.list_files:
         cfg.print_import_time = False
 
     # Set up the python path
@@ -712,6 +1012,19 @@ def main(argv):
     logging.basicConfig()
     logging.root.setLevel(logging.CRITICAL)
 
+    # Configure doctests
+    if cfg.first_doctest_failure:
+        import doctest
+        # The doctest module in Python 2.3 does not have this feature
+        if hasattr(doctest, 'REPORT_ONLY_FIRST_FAILURE'):
+            doctest.set_unittest_reportflags(doctest.REPORT_ONLY_FIRST_FAILURE)
+        # Also apply the flag to zope.testing.doctest, if it exists
+        try:
+            from zope.testing import doctest
+            doctest.set_unittest_reportflags(doctest.REPORT_ONLY_FIRST_FAILURE)
+        except ImportError:
+            pass
+
     # Running
     success = True
     if cfg.list_files:
